@@ -116,13 +116,13 @@
 
 /* Parse a string into an internal time stamp.
 
-   Copyright (C) 1999, 2000, 2002, 2003, 2004, 2005, 2006 Free Software
+   Copyright (C) 1999, 2000, 2002, 2003, 2004, 2005, 2006, 2007 Free Software
    Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -130,8 +130,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Originally written by Steven M. Bellovin <smb@research.att.com> while
    at the University of North Carolina at Chapel Hill.  Later tweaked by
@@ -151,6 +150,10 @@
 
 #include "getdate.h"
 
+#include "intprops.h"
+#include "timespec.h"
+#include "verify.h"
+
 /* There's no need to extend the stack, so there's no need to involve
    alloca.  */
 #define YYSTACK_USE_ALLOCA 0
@@ -221,6 +224,13 @@
 
 #define HOUR(x) ((x) * 60)
 
+/* Lots of this code assumes time_t and time_t-like values fit into
+   long int.  It also assumes that signed integer overflow silently
+   wraps around, but there's no portable way to check for that at
+   compile-time.  */
+verify (TYPE_IS_INTEGER (time_t));
+verify (LONG_MIN <= TYPE_MINIMUM (time_t) && TYPE_MAXIMUM (time_t) <= LONG_MAX);
+
 /* An integer value, and the number of digits in its textual
    representation.  */
 typedef struct
@@ -310,7 +320,7 @@ typedef struct
 
 union YYSTYPE;
 static int yylex (union YYSTYPE *, parser_control *);
-static int yyerror (parser_control *, char *);
+static int yyerror (parser_control const *, char const *);
 static long int time_zone_hhmm (textint, long int);
 
 
@@ -335,15 +345,15 @@ static long int time_zone_hhmm (textint, long int);
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 213 "getdate.y"
+#line 223 "getdate.y"
 {
   long int intval;
   textint textintval;
   struct timespec timespec;
   relative_time rel;
 }
-/* Line 193 of yacc.c.  */
-#line 347 "getdate.c"
+/* Line 187 of yacc.c.  */
+#line 357 "getdate.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -356,7 +366,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 360 "getdate.c"
+#line 370 "getdate.c"
 
 #ifdef short
 # undef short
@@ -668,14 +678,14 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   239,   239,   240,   244,   251,   253,   257,   259,   261,
-     263,   265,   267,   269,   273,   281,   289,   299,   306,   318,
-     323,   331,   333,   343,   345,   347,   352,   357,   362,   367,
-     375,   380,   400,   407,   415,   423,   428,   434,   439,   448,
-     458,   471,   473,   475,   477,   479,   481,   483,   485,   487,
-     489,   491,   493,   495,   497,   499,   501,   503,   505,   507,
-     509,   511,   515,   517,   519,   521,   523,   525,   529,   529,
-     532,   533,   538,   539,   544,   582,   583,   589,   590
+       0,   249,   249,   250,   254,   261,   263,   267,   269,   271,
+     273,   275,   277,   279,   283,   291,   299,   309,   316,   328,
+     333,   341,   343,   353,   355,   357,   362,   367,   372,   377,
+     385,   390,   410,   417,   425,   433,   438,   444,   449,   458,
+     468,   481,   483,   485,   487,   489,   491,   493,   495,   497,
+     499,   501,   503,   505,   507,   509,   511,   513,   515,   517,
+     519,   521,   525,   527,   529,   531,   533,   535,   539,   539,
+     542,   543,   548,   549,   554,   592,   593,   599,   600
 };
 #endif
 
@@ -1647,7 +1657,7 @@ yyreduce:
   switch (yyn)
     {
         case 4:
-#line 245 "getdate.y"
+#line 255 "getdate.y"
     {
 	pc->seconds = (yyvsp[(2) - (2)].timespec);
 	pc->timespec_seen = true;
@@ -1655,37 +1665,37 @@ yyreduce:
     break;
 
   case 7:
-#line 258 "getdate.y"
+#line 268 "getdate.y"
     { pc->times_seen++; }
     break;
 
   case 8:
-#line 260 "getdate.y"
+#line 270 "getdate.y"
     { pc->local_zones_seen++; }
     break;
 
   case 9:
-#line 262 "getdate.y"
+#line 272 "getdate.y"
     { pc->zones_seen++; }
     break;
 
   case 10:
-#line 264 "getdate.y"
+#line 274 "getdate.y"
     { pc->dates_seen++; }
     break;
 
   case 11:
-#line 266 "getdate.y"
+#line 276 "getdate.y"
     { pc->days_seen++; }
     break;
 
   case 12:
-#line 268 "getdate.y"
+#line 278 "getdate.y"
     { pc->rels_seen = true; }
     break;
 
   case 14:
-#line 274 "getdate.y"
+#line 284 "getdate.y"
     {
 	pc->hour = (yyvsp[(1) - (2)].textintval).value;
 	pc->minutes = 0;
@@ -1696,7 +1706,7 @@ yyreduce:
     break;
 
   case 15:
-#line 282 "getdate.y"
+#line 292 "getdate.y"
     {
 	pc->hour = (yyvsp[(1) - (4)].textintval).value;
 	pc->minutes = (yyvsp[(3) - (4)].textintval).value;
@@ -1707,7 +1717,7 @@ yyreduce:
     break;
 
   case 16:
-#line 290 "getdate.y"
+#line 300 "getdate.y"
     {
 	pc->hour = (yyvsp[(1) - (5)].textintval).value;
 	pc->minutes = (yyvsp[(3) - (5)].textintval).value;
@@ -1720,7 +1730,7 @@ yyreduce:
     break;
 
   case 17:
-#line 300 "getdate.y"
+#line 310 "getdate.y"
     {
 	pc->hour = (yyvsp[(1) - (6)].textintval).value;
 	pc->minutes = (yyvsp[(3) - (6)].textintval).value;
@@ -1730,7 +1740,7 @@ yyreduce:
     break;
 
   case 18:
-#line 307 "getdate.y"
+#line 317 "getdate.y"
     {
 	pc->hour = (yyvsp[(1) - (7)].textintval).value;
 	pc->minutes = (yyvsp[(3) - (7)].textintval).value;
@@ -1742,7 +1752,7 @@ yyreduce:
     break;
 
   case 19:
-#line 319 "getdate.y"
+#line 329 "getdate.y"
     {
 	pc->local_isdst = (yyvsp[(1) - (1)].intval);
 	pc->dsts_seen += (0 < (yyvsp[(1) - (1)].intval));
@@ -1750,7 +1760,7 @@ yyreduce:
     break;
 
   case 20:
-#line 324 "getdate.y"
+#line 334 "getdate.y"
     {
 	pc->local_isdst = 1;
 	pc->dsts_seen += (0 < (yyvsp[(1) - (2)].intval)) + 1;
@@ -1758,12 +1768,12 @@ yyreduce:
     break;
 
   case 21:
-#line 332 "getdate.y"
+#line 342 "getdate.y"
     { pc->time_zone = (yyvsp[(1) - (1)].intval); }
     break;
 
   case 22:
-#line 334 "getdate.y"
+#line 344 "getdate.y"
     { pc->time_zone = (yyvsp[(1) - (2)].intval);
 	pc->rel.ns += (yyvsp[(2) - (2)].rel).ns;
 	pc->rel.seconds += (yyvsp[(2) - (2)].rel).seconds;
@@ -1776,22 +1786,22 @@ yyreduce:
     break;
 
   case 23:
-#line 344 "getdate.y"
+#line 354 "getdate.y"
     { pc->time_zone = (yyvsp[(1) - (3)].intval) + time_zone_hhmm ((yyvsp[(2) - (3)].textintval), (yyvsp[(3) - (3)].intval)); }
     break;
 
   case 24:
-#line 346 "getdate.y"
+#line 356 "getdate.y"
     { pc->time_zone = (yyvsp[(1) - (1)].intval) + 60; }
     break;
 
   case 25:
-#line 348 "getdate.y"
+#line 358 "getdate.y"
     { pc->time_zone = (yyvsp[(1) - (2)].intval) + 60; }
     break;
 
   case 26:
-#line 353 "getdate.y"
+#line 363 "getdate.y"
     {
 	pc->day_ordinal = 1;
 	pc->day_number = (yyvsp[(1) - (1)].intval);
@@ -1799,7 +1809,7 @@ yyreduce:
     break;
 
   case 27:
-#line 358 "getdate.y"
+#line 368 "getdate.y"
     {
 	pc->day_ordinal = 1;
 	pc->day_number = (yyvsp[(1) - (2)].intval);
@@ -1807,7 +1817,7 @@ yyreduce:
     break;
 
   case 28:
-#line 363 "getdate.y"
+#line 373 "getdate.y"
     {
 	pc->day_ordinal = (yyvsp[(1) - (2)].intval);
 	pc->day_number = (yyvsp[(2) - (2)].intval);
@@ -1815,7 +1825,7 @@ yyreduce:
     break;
 
   case 29:
-#line 368 "getdate.y"
+#line 378 "getdate.y"
     {
 	pc->day_ordinal = (yyvsp[(1) - (2)].textintval).value;
 	pc->day_number = (yyvsp[(2) - (2)].intval);
@@ -1823,7 +1833,7 @@ yyreduce:
     break;
 
   case 30:
-#line 376 "getdate.y"
+#line 386 "getdate.y"
     {
 	pc->month = (yyvsp[(1) - (3)].textintval).value;
 	pc->day = (yyvsp[(3) - (3)].textintval).value;
@@ -1831,7 +1841,7 @@ yyreduce:
     break;
 
   case 31:
-#line 381 "getdate.y"
+#line 391 "getdate.y"
     {
 	/* Interpret as YYYY/MM/DD if the first value has 4 or more digits,
 	   otherwise as MM/DD/YY.
@@ -1854,7 +1864,7 @@ yyreduce:
     break;
 
   case 32:
-#line 401 "getdate.y"
+#line 411 "getdate.y"
     {
 	/* ISO 8601 format.  YYYY-MM-DD.  */
 	pc->year = (yyvsp[(1) - (3)].textintval);
@@ -1864,7 +1874,7 @@ yyreduce:
     break;
 
   case 33:
-#line 408 "getdate.y"
+#line 418 "getdate.y"
     {
 	/* e.g. 17-JUN-1992.  */
 	pc->day = (yyvsp[(1) - (3)].textintval).value;
@@ -1875,7 +1885,7 @@ yyreduce:
     break;
 
   case 34:
-#line 416 "getdate.y"
+#line 426 "getdate.y"
     {
 	/* e.g. JUN-17-1992.  */
 	pc->month = (yyvsp[(1) - (3)].intval);
@@ -1886,7 +1896,7 @@ yyreduce:
     break;
 
   case 35:
-#line 424 "getdate.y"
+#line 434 "getdate.y"
     {
 	pc->month = (yyvsp[(1) - (2)].intval);
 	pc->day = (yyvsp[(2) - (2)].textintval).value;
@@ -1894,7 +1904,7 @@ yyreduce:
     break;
 
   case 36:
-#line 429 "getdate.y"
+#line 439 "getdate.y"
     {
 	pc->month = (yyvsp[(1) - (4)].intval);
 	pc->day = (yyvsp[(2) - (4)].textintval).value;
@@ -1903,7 +1913,7 @@ yyreduce:
     break;
 
   case 37:
-#line 435 "getdate.y"
+#line 445 "getdate.y"
     {
 	pc->day = (yyvsp[(1) - (2)].textintval).value;
 	pc->month = (yyvsp[(2) - (2)].intval);
@@ -1911,7 +1921,7 @@ yyreduce:
     break;
 
   case 38:
-#line 440 "getdate.y"
+#line 450 "getdate.y"
     {
 	pc->day = (yyvsp[(1) - (3)].textintval).value;
 	pc->month = (yyvsp[(2) - (3)].intval);
@@ -1920,7 +1930,7 @@ yyreduce:
     break;
 
   case 39:
-#line 449 "getdate.y"
+#line 459 "getdate.y"
     {
 	pc->rel.ns -= (yyvsp[(1) - (2)].rel).ns;
 	pc->rel.seconds -= (yyvsp[(1) - (2)].rel).seconds;
@@ -1933,7 +1943,7 @@ yyreduce:
     break;
 
   case 40:
-#line 459 "getdate.y"
+#line 469 "getdate.y"
     {
 	pc->rel.ns += (yyvsp[(1) - (1)].rel).ns;
 	pc->rel.seconds += (yyvsp[(1) - (1)].rel).seconds;
@@ -1946,147 +1956,147 @@ yyreduce:
     break;
 
   case 41:
-#line 472 "getdate.y"
+#line 482 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).year = (yyvsp[(1) - (2)].intval); }
     break;
 
   case 42:
-#line 474 "getdate.y"
+#line 484 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).year = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 43:
-#line 476 "getdate.y"
+#line 486 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).year = 1; }
     break;
 
   case 44:
-#line 478 "getdate.y"
+#line 488 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).month = (yyvsp[(1) - (2)].intval); }
     break;
 
   case 45:
-#line 480 "getdate.y"
+#line 490 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).month = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 46:
-#line 482 "getdate.y"
+#line 492 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).month = 1; }
     break;
 
   case 47:
-#line 484 "getdate.y"
+#line 494 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).day = (yyvsp[(1) - (2)].intval) * (yyvsp[(2) - (2)].intval); }
     break;
 
   case 48:
-#line 486 "getdate.y"
+#line 496 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).day = (yyvsp[(1) - (2)].textintval).value * (yyvsp[(2) - (2)].intval); }
     break;
 
   case 49:
-#line 488 "getdate.y"
+#line 498 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).day = (yyvsp[(1) - (1)].intval); }
     break;
 
   case 50:
-#line 490 "getdate.y"
+#line 500 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).hour = (yyvsp[(1) - (2)].intval); }
     break;
 
   case 51:
-#line 492 "getdate.y"
+#line 502 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).hour = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 52:
-#line 494 "getdate.y"
+#line 504 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).hour = 1; }
     break;
 
   case 53:
-#line 496 "getdate.y"
+#line 506 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).minutes = (yyvsp[(1) - (2)].intval); }
     break;
 
   case 54:
-#line 498 "getdate.y"
+#line 508 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).minutes = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 55:
-#line 500 "getdate.y"
+#line 510 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).minutes = 1; }
     break;
 
   case 56:
-#line 502 "getdate.y"
+#line 512 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = (yyvsp[(1) - (2)].intval); }
     break;
 
   case 57:
-#line 504 "getdate.y"
+#line 514 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 58:
-#line 506 "getdate.y"
+#line 516 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = (yyvsp[(1) - (2)].timespec).tv_sec; (yyval.rel).ns = (yyvsp[(1) - (2)].timespec).tv_nsec; }
     break;
 
   case 59:
-#line 508 "getdate.y"
+#line 518 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = (yyvsp[(1) - (2)].timespec).tv_sec; (yyval.rel).ns = (yyvsp[(1) - (2)].timespec).tv_nsec; }
     break;
 
   case 60:
-#line 510 "getdate.y"
+#line 520 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = 1; }
     break;
 
   case 62:
-#line 516 "getdate.y"
+#line 526 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).year = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 63:
-#line 518 "getdate.y"
+#line 528 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).month = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 64:
-#line 520 "getdate.y"
+#line 530 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).day = (yyvsp[(1) - (2)].textintval).value * (yyvsp[(2) - (2)].intval); }
     break;
 
   case 65:
-#line 522 "getdate.y"
+#line 532 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).hour = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 66:
-#line 524 "getdate.y"
+#line 534 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).minutes = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 67:
-#line 526 "getdate.y"
+#line 536 "getdate.y"
     { (yyval.rel) = RELATIVE_TIME_0; (yyval.rel).seconds = (yyvsp[(1) - (2)].textintval).value; }
     break;
 
   case 71:
-#line 534 "getdate.y"
+#line 544 "getdate.y"
     { (yyval.timespec).tv_sec = (yyvsp[(1) - (1)].textintval).value; (yyval.timespec).tv_nsec = 0; }
     break;
 
   case 73:
-#line 540 "getdate.y"
+#line 550 "getdate.y"
     { (yyval.timespec).tv_sec = (yyvsp[(1) - (1)].textintval).value; (yyval.timespec).tv_nsec = 0; }
     break;
 
   case 74:
-#line 545 "getdate.y"
+#line 555 "getdate.y"
     {
 	if (pc->dates_seen && ! pc->year.digits
 	    && ! pc->rels_seen && (pc->times_seen || 2 < (yyvsp[(1) - (1)].textintval).digits))
@@ -2123,28 +2133,28 @@ yyreduce:
     break;
 
   case 75:
-#line 582 "getdate.y"
+#line 592 "getdate.y"
     { (yyval.intval) = -1; }
     break;
 
   case 76:
-#line 584 "getdate.y"
+#line 594 "getdate.y"
     { (yyval.intval) = (yyvsp[(2) - (2)].textintval).value; }
     break;
 
   case 77:
-#line 589 "getdate.y"
+#line 599 "getdate.y"
     { (yyval.intval) = MER24; }
     break;
 
   case 78:
-#line 591 "getdate.y"
+#line 601 "getdate.y"
     { (yyval.intval) = (yyvsp[(1) - (1)].intval); }
     break;
 
 
 /* Line 1267 of yacc.c.  */
-#line 2148 "getdate.c"
+#line 2158 "getdate.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2358,7 +2368,7 @@ yyreturn:
 }
 
 
-#line 594 "getdate.y"
+#line 604 "getdate.y"
 
 
 static table const meridian_table[] =
@@ -2874,7 +2884,8 @@ yylex (YYSTYPE *lvalp, parser_control *pc)
 
 /* Do nothing if the parser reports an error.  */
 static int
-yyerror (parser_control *pc ATTRIBUTE_UNUSED, char *s ATTRIBUTE_UNUSED)
+yyerror (parser_control const *pc ATTRIBUTE_UNUSED,
+	 char const *s ATTRIBUTE_UNUSED)
 {
   return 0;
 }
@@ -2995,6 +3006,12 @@ get_date (struct timespec *result, char const *p, struct timespec const *now)
 	  }
     }
 
+  /* As documented, be careful to treat the empty string just like
+     a date string of "0".  Without this, an empty string would be
+     declared invalid when parsed during a DST transition.  */
+  if (*p == '\0')
+    p = "0";
+
   pc.input = p;
   pc.year.value = tmp->tm_year;
   pc.year.value += TM_YEAR_BASE;
