@@ -1,9 +1,10 @@
 /* extern.h - External declarations for cpio.  Requires system.h.
-   Copyright (C) 1990, 1991, 1992, 2001, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1990, 1991, 1992, 2001, 2006, 
+   2007 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,6 +17,10 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.  */
 
+#include "paxlib.h"
+#include "quotearg.h"
+#include "quote.h"
+
 enum archive_format
 {
   arf_unknown, arf_binary, arf_oldascii, arf_newascii, arf_crcascii,
@@ -50,10 +55,12 @@ extern int quiet_flag;
 extern int only_verify_crc_flag;
 extern int no_abs_paths_flag;
 extern unsigned int warn_option;
+extern mode_t newdir_umask;
 
 /* Values for warn_option */
 #define CPIO_WARN_NONE     0
 #define CPIO_WARN_TRUNCATE 0x01
+#define CPIO_WARN_INTERDIR 0x02
 #define CPIO_WARN_ALL      (unsigned int)-1
 
 extern bool to_stdout_option;
@@ -76,13 +83,10 @@ extern int debug_flag;
 
 extern char *input_buffer, *output_buffer;
 extern char *in_buff, *out_buff;
-extern long input_buffer_size;
-extern long input_size, output_size;
-#ifdef __GNUC__
-extern long long input_bytes, output_bytes;
-#else
-extern long input_bytes, output_bytes;
-#endif
+extern size_t input_buffer_size;
+extern size_t input_size, output_size;
+extern off_t input_bytes, output_bytes;
+
 extern char *directory_name;
 extern char **save_patterns;
 extern int num_patterns;
@@ -126,20 +130,18 @@ char *dirname (char *path);
 void mode_string (unsigned int mode, char *str);
 
 /* idcache.c */
-#ifndef __MSDOS__
-char *getgroup ();
-char *getuser ();
-uid_t *getuidbyname ();
-gid_t *getgidbyname ();
-#endif
+char *getgroup (gid_t gid);
+char *getuser (uid_t uid);
+uid_t *getuidbyname (char *user);
+gid_t *getgidbyname (char *group);
 
 /* main.c */
 void process_args (int argc, char *argv[]);
 void initialize_buffers (void);
 
 /* makepath.c */
-int make_path (char *argpath, int mode, int parent_mode,
-	       uid_t owner, gid_t group, char *verbose_fmt_string);
+int make_path (char *argpath, uid_t owner, gid_t group,
+	       const char *verbose_fmt_string);
 
 /* tar.c */
 void write_out_tar_header (struct cpio_file_stat *file_hdr, int out_des);
@@ -150,10 +152,8 @@ int is_tar_header (char *buf);
 int is_tar_filename_too_long (char *name);
 
 /* userspec.c */
-#ifndef __MSDOS__
 char *parse_user_spec (char *name, uid_t *uid, gid_t *gid,
 		       char **username, char **groupname);
-#endif
 
 /* util.c */
 void tape_empty_output_buffer (int out_des);
@@ -162,12 +162,12 @@ void swahw_array (char *ptr, int count);
 void tape_buffered_write (char *in_buf, int out_des, off_t num_bytes);
 void tape_buffered_read (char *in_buf, int in_des, off_t num_bytes);
 int tape_buffered_peek (char *peek_buf, int in_des, int num_bytes);
-void tape_toss_input (int in_des, long num_bytes);
+void tape_toss_input (int in_des, off_t num_bytes);
 void copy_files_tape_to_disk (int in_des, int out_des, off_t num_bytes);
 void copy_files_disk_to_tape (int in_des, int out_des, off_t num_bytes, char *filename);
 void copy_files_disk_to_disk (int in_des, int out_des, off_t num_bytes, char *filename);
 void warn_if_file_changed (char *file_name, unsigned long old_file_size,
-                           unsigned long old_file_mtime);
+                           time_t old_file_mtime);
 void create_all_directories (char *name);
 void prepare_append (int out_file_des);
 char *find_inode_file (unsigned long node_num,
@@ -178,12 +178,6 @@ int open_archive (char *file);
 void tape_offline (int tape_des);
 void get_next_reel (int tape_des);
 void set_new_media_message (char *message);
-#if defined(__MSDOS__) && !defined(__GNUC__)
-int chown (char *path, int owner, int group);
-#endif
-#ifdef __TURBOC__
-int utime (char *filename, struct utimbuf *utb);
-#endif
 #ifdef HPUX_CDF
 char *add_cdf_double_slashes (char *filename);
 #endif
@@ -199,8 +193,9 @@ void write_nuls_to_file (off_t num_bytes, int out_des,
 # define UMASKED_SYMLINK(name1,name2,mode)    umasked_symlink(name1,name2,mode)
 #endif /* SYMLINK_USES_UMASK */
 
-void set_perms (struct cpio_file_stat *header);
-void set_file_times (const char *name, unsigned long atime, unsigned long mtime);
+void set_perms (int fd, struct cpio_file_stat *header);
+void set_file_times (int fd, const char *name, unsigned long atime,
+		     unsigned long mtime);
 void stat_to_cpio (struct cpio_file_stat *hdr, struct stat *st);
 void cpio_safer_name_suffix (char *name, bool link_target,
 			     bool absolute_names, bool strip_leading_dots);
@@ -214,3 +209,9 @@ uintmax_t from_ascii (char const *where, size_t digs, unsigned logbase);
 #define FROM_OCTAL(f) from_ascii (f, sizeof f, LG_8)
 #define FROM_HEX(f) from_ascii (f, sizeof f, LG_16)
 	    
+void delay_set_stat (char const *file_name, struct stat *st,
+		     mode_t invert_permissions);
+void repair_delayed_set_stat (char const *dir,
+			      struct stat *dir_stat_info);
+void apply_delayed_set_stat (void);
+     
