@@ -33,7 +33,9 @@
 #include <hash.h>
 #include <utimens.h>
 
-#include <sys/ioctl.h>
+#ifdef HAVE_SYS_IOCTL_H
+# include <sys/ioctl.h>
+#endif
 
 #ifdef HAVE_SYS_MTIO_H
 # ifdef HAVE_SYS_IO_TRIOCTL_H
@@ -441,8 +443,8 @@ write_nuls_to_file (off_t num_bytes, int out_des,
 void
 copy_files_tape_to_disk (int in_des, int out_des, off_t num_bytes)
 {
-  long size;
-  long k;
+  off_t size;
+  off_t k;
 
   while (num_bytes > 0)
     {
@@ -472,8 +474,8 @@ void
 copy_files_disk_to_tape (int in_des, int out_des, off_t num_bytes,
 			 char *filename)
 {
-  long size;
-  long k;
+  off_t size;
+  off_t k;
   int rc;
   off_t original_num_bytes;
 
@@ -568,7 +570,7 @@ copy_files_disk_to_disk (int in_des, int out_des, off_t num_bytes,
 
 void
 warn_if_file_changed (char *file_name, unsigned long old_file_size,
-		      off_t old_file_mtime)
+		      time_t old_file_mtime)
 {
   struct stat new_file_stat;
   if ((*xstat) (file_name, &new_file_stat) < 0)
@@ -1266,7 +1268,10 @@ stat_to_cpio (struct cpio_file_stat *hdr, struct stat *st)
 }
 
 #ifndef HAVE_FCHOWN
-# define fchown(fd, uid, gid) (-1)
+# define HAVE_FCHOWN 0
+#endif
+#ifndef HAVE_FCHMOD
+# define HAVE_FCHMOD 0
 #endif
 
 int
